package com.ectrip.cyt.client.cyt.service;

import com.alibaba.fastjson.JSONObject;

import com.ectrip.cyt.client.cyt.common.Parameter;
import com.ectrip.cyt.client.cyt.model.cytOrder.request.*;
import com.ectrip.cyt.client.cyt.model.cytOrder.response.CreateOrderResponseBody;
import com.ectrip.cyt.client.cyt.model.cytOrder.response.CreatePaymentOrderResponseBody;
import com.ectrip.cyt.client.cyt.model.cytOrder.response.ResponseHeader;
import com.ectrip.cyt.exception.BusinessException;
import com.ectrip.cyt.model.enums.ApiMethodType;
import com.ectrip.cyt.model.param.*;
import com.ectrip.cyt.model.pojo.EditPerson;
import com.ectrip.cyt.model.pojo.Person;
import com.ectrip.cyt.model.result.Result;
import com.ectrip.cyt.model.enums.CredentialsType;
import com.ectrip.cyt.model.enums.OrderStatusType;
import com.ectrip.cyt.model.rt.CommonReturn;
import com.ectrip.cyt.utils.*;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.http.HttpStatus;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.SocketTimeoutException;
import java.nio.charset.Charset;
import java.util.*;

public class TdosApiClient{
    public static Result heartBeat(AccountParam accountParam){
        return commonApi(accountParam,new RequestBody(),OtaUtils.HEART_BEAT);
    }

    public static Result getBalance(AccountParam accountParam) {
        return commonApi(accountParam,new RequestBody(),OtaUtils.GET_BALANCE);
    }

    public static Result checkOrder(CreateOrderParam orderParam, AccountParam accountParam) {
        CreateOrderRequestBody requestBody = new CreateOrderRequestBody();
        CreateOrderRequestBody.OrderInfo requestOrderInfo = new CreateOrderRequestBody.OrderInfo();
        requestOrderInfo.setOrderId(orderParam.getOrderCode());
        //测试接口不支持淘宝
        //requestOrderInfo.setToken();//淘宝订单（令牌信息）
        //产品信息
        CreateOrderRequestBody.OrderInfo.Product product = new CreateOrderRequestBody.OrderInfo.Product();
        product.setResourceId(orderParam.getProduct().getProductNo());  //产品id
        product.setVisitDate(formatDate(DateUtils.convertDate(orderParam.getVisitDate(),DateUtils.DATE_PATTERN)));//游玩日期
        /*if(StringUtils.isNotBlank(orderParam.getLeaveDate())) {
            product.setLeaveDate(formatDate(DateUtils.convertDate(orderParam.getLeaveDate(), DateUtils.DATE_PATTERN)));//住店日期
        }*/
        product.setSellPrice(Long.toString(convertCents(orderParam.getProduct().getSellPrice())));//产品售卖价格，转换为分
        product.setTimeOfBookId(Long.toString(orderParam.getProduct().getTimeOfBookId()));
        requestOrderInfo.setProduct(product);
        //取票人信息
        CreateOrderRequestBody.OrderInfo.ContactPerson contactPerson = new CreateOrderRequestBody.OrderInfo.ContactPerson();
        contactPerson.setName(orderParam.getContactPerson().getName());
        contactPerson.setMobile(orderParam.getContactPerson().getPhone());
        contactPerson.setCredentials(orderParam.getContactPerson().getCredentials());
        CredentialsType credentialsType=CredentialsType.typeOf(orderParam.getContactPerson().getCredentialsType());
        contactPerson.setCredentialsType(OtaUtils.credentialsTypeStringMap.get(credentialsType));
        requestOrderInfo.setContactPerson(contactPerson);
        //游玩人信息
        CreateOrderRequestBody.OrderInfo.VisitPerson visitPerson = new CreateOrderRequestBody.OrderInfo.VisitPerson();
        List<CreateOrderRequestBody.OrderInfo.VisitPerson.Person> persons = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(orderParam.getVisitPersons())) {
            for (Person p : orderParam.getVisitPersons()) {
                CreateOrderRequestBody.OrderInfo.VisitPerson.Person person = new CreateOrderRequestBody.OrderInfo.VisitPerson.Person();
                person.setName(p.getName());
                person.setCredentials(p.getCredentials());
                CredentialsType ct=CredentialsType.typeOf(p.getCredentialsType());
                person.setCredentialsType(OtaUtils.credentialsTypeStringMap.get(ct)
                );
                person.setMobile(p.getPhone());
                persons.add(person);
            }
        }
        visitPerson.setPerson(persons);
        requestOrderInfo.setVisitPerson(visitPerson);
        OrderStatusType orderStatusType=OrderStatusType.ORDER_RESERVE_SUCCESSFUL;
        //订单票数
        requestOrderInfo.setOrderQuantity(orderParam.getOrderQuantity().toString());
        //订单金额
        requestOrderInfo.setOrderPrice(Long.toString(convertCents(orderParam.getOrderPrice())));
        //只支持预付
        requestOrderInfo.setOrderStatus(OtaUtils.prepayOrderStatusTypeStringMap.get(orderStatusType));
        requestBody.setOrderInfo(requestOrderInfo);
        return commonApi(accountParam,requestBody,OtaUtils.CHECK_ORDER);
    }

    public static Result createOrder(CreateOrderParam orderParam, AccountParam accountParam) {
        CreateOrderRequestBody requestBody = new CreateOrderRequestBody();
        CreateOrderRequestBody.OrderInfo requestOrderInfo = new CreateOrderRequestBody.OrderInfo();
        requestOrderInfo.setOrderId(orderParam.getOrderCode());
        //测试接口不支持淘宝
        //requestOrderInfo.setToken();//淘宝订单（令牌信息）
        //产品信息
        CreateOrderRequestBody.OrderInfo.Product product = new CreateOrderRequestBody.OrderInfo.Product();
        product.setResourceId(orderParam.getProduct().getProductNo());  //产品id
        product.setVisitDate(formatDate(DateUtils.convertDate(orderParam.getVisitDate(),DateUtils.DATE_PATTERN)));//游玩日期
        /*if(StringUtils.isNotBlank(orderParam.getLeaveDate())) {
            product.setLeaveDate(formatDate(DateUtils.convertDate(orderParam.getLeaveDate(), DateUtils.DATE_PATTERN)));//住店日期
        }*/
        product.setSellPrice(Long.toString(convertCents(orderParam.getProduct().getSellPrice())));//产品售卖价格，转换为分
        product.setTimeOfBookId(Long.toString(orderParam.getProduct().getTimeOfBookId()));
        //product.setSeatIds(orderParam.getProduct().getSeatIds());
        requestOrderInfo.setProduct(product);
        //取票人信息
        CreateOrderRequestBody.OrderInfo.ContactPerson contactPerson = new CreateOrderRequestBody.OrderInfo.ContactPerson();
        contactPerson.setName(orderParam.getContactPerson().getName());
        contactPerson.setMobile(orderParam.getContactPerson().getPhone());
        //contactPerson.setCredentials(orderParam.getContactPerson().getCredentials());
        CredentialsType credentialsType=CredentialsType.typeOf(orderParam.getContactPerson().getCredentialsType());
        //contactPerson.setCredentialsType(OtaUtils.credentialsTypeStringMap.get(credentialsType));
        requestOrderInfo.setContactPerson(contactPerson);
        //游玩人信息
        CreateOrderRequestBody.OrderInfo.VisitPerson visitPerson = new CreateOrderRequestBody.OrderInfo.VisitPerson();
        List<CreateOrderRequestBody.OrderInfo.VisitPerson.Person> persons = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(orderParam.getVisitPersons())) {
            for (Person p : orderParam.getVisitPersons()) {
                CreateOrderRequestBody.OrderInfo.VisitPerson.Person person = new CreateOrderRequestBody.OrderInfo.VisitPerson.Person();
                person.setName(p.getName());
                person.setCredentials(p.getCredentials());
                CredentialsType ct=CredentialsType.typeOf(p.getCredentialsType());
                person.setCredentialsType(OtaUtils.credentialsTypeStringMap.get(ct)
                );
                person.setMobile(p.getPhone());
                persons.add(person);
            }
        }
        visitPerson.setPerson(persons);
        requestOrderInfo.setVisitPerson(visitPerson);
        OrderStatusType orderStatusType=OrderStatusType.ORDER_NOT_PAYED_YET;
        //订单票数
        requestOrderInfo.setOrderQuantity(orderParam.getOrderQuantity().toString());
        //订单金额
        requestOrderInfo.setOrderPrice(Long.toString(convertCents(orderParam.getOrderPrice())));
        //只支持预付
        requestOrderInfo.setOrderStatus(OtaUtils.prepayOrderStatusTypeStringMap.get(orderStatusType));
        requestBody.setOrderInfo(requestOrderInfo);
        return commonApi(accountParam,requestBody,OtaUtils.CREATE_ORDER);
    }

    public static Result cancelOrder(CancelOrderParam cancelOrder,AccountParam accountParam) {
        CancelOrderRequestBody requestBody=new CancelOrderRequestBody();
        CancelOrderRequestBody.OrderInfo oiRequest = new CancelOrderRequestBody.OrderInfo();
        oiRequest.setPartnerOrderId(cancelOrder.getCytOrderCode());
        oiRequest.setOrderId(cancelOrder.getOtaOrderCode());
        requestBody.setOrderInfo(oiRequest);
        return commonApi(accountParam,requestBody,OtaUtils.CANCEL_ORDER);
    }

    public static Result payOrder(PayOrderParam payOrder,AccountParam accountParam) {
        PayOrderRequestBody requestBody=new PayOrderRequestBody();
        PayOrderRequestBody.OrderInfo oiRequest = new PayOrderRequestBody.OrderInfo();
        oiRequest.setPartnerOrderId(payOrder.getCytOrderCode());
        oiRequest.setOtaOrderId(payOrder.getOtaOrderCode());
        oiRequest.setOrderPrice(Long.toString(convertCents(payOrder.getOrderPrice())));
        oiRequest.setOrderStatus(OrderStatusUtils.PREPAY_ORDER_PRINT_SUCCESS);
        requestBody.setOrderInfo(oiRequest);
        return commonApi(accountParam,requestBody,OtaUtils.PAY_ORDER);
    }

    public static Result queryOrder(QueryOrderParam queryOrder,AccountParam accountParam) {
        GetOrderByOTARequestBody requestBody=new GetOrderByOTARequestBody();
        //ota订单查询
        requestBody.setOrderId(queryOrder.getOrderCode());
        requestBody.setPartnerOrderId(queryOrder.getCytOrderCode());
        return commonApi(accountParam,requestBody,OtaUtils.GET_ORDER_BY_OTA);
    }

    public static Result pushOrder(PushOrderParam pushOrder,AccountParam accountParam) {
        PushOrderRequestBody requestBody=new PushOrderRequestBody();
        PushOrderRequestBody.OrderInfo oiRequest = new PushOrderRequestBody.OrderInfo();
        //联系人信息
        Person contact = pushOrder.getContactPerson();
        if (contact != null) {
            PushOrderRequestBody.OrderInfo.ContactPerson contactPerson = new PushOrderRequestBody.OrderInfo.ContactPerson();
            contactPerson.setName(contact.getName());
            contactPerson.setMobile(contact.getPhone());
            contactPerson.setCredentials(contact.getCredentials());
            CredentialsType ct=CredentialsType.typeOf(contact.getCredentialsType());
            contactPerson.setCredentialsType(OtaUtils.credentialsTypeStringMap.get(ct));
            oiRequest.setContactPerson(contactPerson);
        }
        //游玩人信息
        if (CollectionUtils.isNotEmpty(pushOrder.getVisitPersons())) {
            PushOrderRequestBody.OrderInfo.VisitPerson visitPerson = new PushOrderRequestBody.OrderInfo.VisitPerson();
            List<PushOrderRequestBody.OrderInfo.VisitPerson.Person> persons = Lists.newArrayList();
            for (EditPerson visitor : pushOrder.getVisitPersons()) {
                PushOrderRequestBody.OrderInfo.VisitPerson.Person person = new PushOrderRequestBody.OrderInfo.VisitPerson.Person();
                person.setCredentials(visitor.getEditCredentials());
                person.setName(visitor.getEditName());
                persons.add(person);
            }
            visitPerson.setPerson(persons);
            oiRequest.setVisitPerson(visitPerson);
        }
        oiRequest.setVisitDate(pushOrder.getVisitDate());
        oiRequest.setPartnerOrderId(pushOrder.getCytOrderCode());
        requestBody.setOrderInfo(oiRequest);
        return commonApi(accountParam,requestBody,OtaUtils.PUSH_ORDER);
    }

    public static Result applyOrderRefundByUser(RefundOrderParam refundOrder,AccountParam accountParam) {
        ApplyOrderRefundByUserRequestBody requestBody=new ApplyOrderRefundByUserRequestBody();
        ApplyOrderRefundByUserRequestBody.OrderInfo oiRequest = new ApplyOrderRefundByUserRequestBody.OrderInfo();
        //畅游通订单号
        oiRequest.setPartnerOrderId(refundOrder.getCytOrderCode());
        //退款流水号
        oiRequest.setRefundSeq(refundOrder.getRefundSeq());
        //订单金额
        //oiRequest.setOrderPrice(Long.toString(convertCents(refundOrder.getOrderMoney())));
        //订单票数
        //oiRequest.setOrderQuantity(Integer.toString(refundOrder.getOrderQuantity()));
        //退款票数
        oiRequest.setRefundQuantity(Integer.toString(refundOrder.getRefundQuantity()));
        //订单退订金额，转换成分
        oiRequest.setOrderRefundPrice(Long.toString(convertCents(refundOrder.getRefundMoney())));
        //退订手续费，转换成分
        //oiRequest.setOrderRefundCharge(Long.toString(convertCents(refundOrder.getRefundCharge())));

        if (CollectionUtils.isNotEmpty(refundOrder.getVisitPersons())) {
            ApplyOrderRefundByUserRequestBody.OrderInfo.VisitPerson visitPerson = new ApplyOrderRefundByUserRequestBody.OrderInfo.VisitPerson();
            List<ApplyOrderRefundByUserRequestBody.OrderInfo.VisitPerson.Person> persons = Lists.newArrayList();
            for (Person visitor : refundOrder.getVisitPersons()) {
                ApplyOrderRefundByUserRequestBody.OrderInfo.VisitPerson.Person person = new ApplyOrderRefundByUserRequestBody.OrderInfo.VisitPerson.Person();
                person.setName(visitor.getName());
                CredentialsType ct=CredentialsType.typeOf(visitor.getCredentialsType());
                person.setCredentialsType(OtaUtils.credentialsTypeStringMap.get(ct));
                person.setCredentials(visitor.getCredentials());
                person.setVoucherCode(visitor.getVoucher());
                persons.add(person);
            }
            visitPerson.setPerson(persons);
            oiRequest.setVisitPerson(visitPerson);
        }

        String refundReason = refundOrder.getRefundReason();
        if (org.apache.commons.lang3.StringUtils.isBlank(refundReason)) {
            refundReason = "客人想退款";
        }
        oiRequest.setRefundExplain(refundReason);
        requestBody.setOrderInfo(oiRequest);
        return commonApi(accountParam,requestBody,OtaUtils.APPLY_ORDER_REFUND_BY_USER);
    }

    public static Result queryPois(AccountParam accountParam){
        return commonApi(accountParam,new RequestBody(),OtaUtils.GET_AVAILABLE_PRODUCT);
    }

    public static Result queryStockAndTsds(GetStockParam param,AccountParam accountParam){
        GetStockProductInfoRequestBody requestBody=new GetStockProductInfoRequestBody();
        requestBody.setResourceId(param.getResourceId());
        requestBody.setStartDate(param.getStartDate());
        requestBody.setEndDate(param.getEndDate());
        return commonApi(accountParam,requestBody,OtaUtils.GET_STOCK_PRODUCT_INFO);
    }

    /*public static Result querySeatList(GetSeatParam param,AccountParam accountParam){
        GetSeatInfoRequestBody requestBody=new GetSeatInfoRequestBody();
        requestBody.setTimeOfBookId(param.getTimeOfBookId());
        return commonApi(accountParam,requestBody,OtaUtils.GET_SEAT_INFO);
    }*/

    public static Result sendOrderEticket(SendOrderEticketParam param,AccountParam accountParam){
        SendOrderEticketRequestBody requestBody=new SendOrderEticketRequestBody();
        requestBody.getOrderInfo().setPartnerOrderId(param.getCytOrderCode());
        requestBody.getOrderInfo().setPhoneNumber(param.getPhone());
        return commonApi(accountParam,requestBody,OtaUtils.SEND_ORDER_ETICKET);
    }

    public static Result queryProducts(GetProductParam productParam, AccountParam accountParam) {
        GetProductByOTARequestBody requestBody=new GetProductByOTARequestBody();
        if(StringUtils.isNotBlank(productParam.getResourceId())){
            requestBody.setResourceId(productParam.getResourceId());
            requestBody.setCurrentPage("");
            requestBody.setPageSize("");
        }else{
            requestBody.setCurrentPage(productParam.getCurrentPage().toString());
            requestBody.setPageSize(productParam.getPageSize().toString());
        }
        return commonApi(accountParam,requestBody,OtaUtils.GET_PRODUCT_BY_OTA);
    }

    public static Result createPaymentOrder(CreateOrderParam orderParam, AccountParam accountParam) {
        CreatePaymentOrderRequestBody requestBody = new CreatePaymentOrderRequestBody();
        CreatePaymentOrderRequestBody.OrderInfo requestOrderInfo = new CreatePaymentOrderRequestBody.OrderInfo();
        requestOrderInfo.setOrderId(orderParam.getOrderCode());
        //产品信息
        CreatePaymentOrderRequestBody.OrderInfo.Product product = new CreatePaymentOrderRequestBody.OrderInfo.Product();
        product.setResourceId(orderParam.getProduct().getProductNo());  //产品id
        product.setVisitDate(formatDate(DateUtils.convertDate(orderParam.getVisitDate(),DateUtils.DATE_PATTERN)));  //游玩日期
        if(StringUtils.isNotBlank(orderParam.getLeaveDate())) {
            product.setLeaveDate(formatDate(DateUtils.convertDate(orderParam.getLeaveDate(), DateUtils.DATE_PATTERN)));//住店日期
        }
        product.setSellPrice(Long.toString(convertCents(orderParam.getProduct().getSellPrice())));//产品售卖价格，转换为分
        product.setTimeOfBookId(Long.toString(orderParam.getProduct().getTimeOfBookId()));
        //product.setSeatIds(orderParam.getProduct().getSeatIds());
        requestOrderInfo.setProduct(product);
        //取票人信息
        CreatePaymentOrderRequestBody.OrderInfo.ContactPerson contactPerson = new CreatePaymentOrderRequestBody.OrderInfo.ContactPerson();
        contactPerson.setName(orderParam.getContactPerson().getName());
        contactPerson.setMobile(orderParam.getContactPerson().getPhone());
        contactPerson.setCredentials(orderParam.getContactPerson().getCredentials());
        CredentialsType credentialsType=CredentialsType.typeOf(orderParam.getContactPerson().getCredentialsType());
        contactPerson.setCredentialsType(OtaUtils.credentialsTypeStringMap.get(credentialsType)
        );
        requestOrderInfo.setContactPerson(contactPerson);
        //游玩人信息
        CreatePaymentOrderRequestBody.OrderInfo.VisitPerson visitPerson = new CreatePaymentOrderRequestBody.OrderInfo.VisitPerson();
        List<CreatePaymentOrderRequestBody.OrderInfo.VisitPerson.Person> persons = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(orderParam.getVisitPersons())) {
            for (Person p : orderParam.getVisitPersons()) {
                CreatePaymentOrderRequestBody.OrderInfo.VisitPerson.Person person = new CreatePaymentOrderRequestBody.OrderInfo.VisitPerson.Person();
                person.setName(p.getName());
                person.setMobile(p.getPhone());
                person.setCredentials(p.getCredentials());
                CredentialsType ct=CredentialsType.typeOf(p.getCredentialsType());
                person.setCredentialsType(OtaUtils.credentialsTypeStringMap.get(ct)
                );
                persons.add(person);
            }
        }
        visitPerson.setPerson(persons);
        requestOrderInfo.setVisitPerson(visitPerson);
        OrderStatusType orderStatusType=OrderStatusType.ORDER_RESERVE_SUCCESSFUL;
        //订单票数
        requestOrderInfo.setOrderQuantity(orderParam.getOrderQuantity().toString());
        //订单金额
        requestOrderInfo.setOrderPrice(Long.toString(convertCents(orderParam.getOrderPrice())));
        //只支持预付
        requestOrderInfo.setOrderStatus(OtaUtils.prepayOrderStatusTypeStringMap.get(orderStatusType));
        requestBody.setOrderInfo(requestOrderInfo);
        return commonApi(accountParam,requestBody,OtaUtils.CREATE_PAY_ORDER);
    }

    //<!=============================通用请求====================================!>
    public static Result commonApi(AccountParam account,RequestBody requestBody,String method){
        ApiMethodType methodType=ApiMethodType.typeOf(method);

        CommonReturn data=new CommonReturn();

        Date dateTimeStart = new Date(); //开始时间
        data.setRequestTime(DateUtils.formatDate(dateTimeStart,DateUtils.DATETIME_PATTERN));

        Map<String, Object> mapRequest = parseRequest(requestBody,account.getDistributorName(), account.getIdentity(), account.getKey(),account.getSecurityType());

        //定义请求的报文
        String requestXml = (String) mapRequest.get(OtaUtils.REQUEST_XML);
        JSONObject requestJson = (JSONObject) mapRequest.get(OtaUtils.REQUEST_JSON_OBJECT);

        Result<String> resultSend = sendData(account.getUrl(), method,
                requestJson.toJSONString());
        if (!resultSend.isSuccess()) {
            return Result.error(resultSend.getThrowable(), resultSend.getDescription());
        }

        String response = resultSend.getData();

        //返回报文
        String responseXml = getResponseXml(response,account.getKey(),account.getSecurityType());

        Date dateTimeEnd = new Date(); //结束时间
        data.setResponseTime(DateUtils.formatDate(dateTimeEnd,DateUtils.DATETIME_PATTERN));
        //记录日志
        data.setRequestJson(requestJson);
        data.setRequestXml(requestXml);

        JSONObject res=JSONObject.parseObject(response);
        if(res.get("xml")!=null){
            res.remove("xml");
        }
        data.setResponseJson(res);
        data.setResponseXml(responseXml);
        data.setInterfaceType("整合营销接口");
        data.setInterfaceMethod(method);
        data.setDescription(methodType.getName());
        //解析报文
        Map map = xmlToBean(responseXml, new ResponseHeader(), new CreateOrderResponseBody());
        ResponseHeader header = (ResponseHeader) map.get(OtaUtils.DOM_HEADER);
        // 判定header返回值，是否为1000
        String codeResponse = header.getCode();
        if (!OtaUtils.SUCCESS_CODE.equals(codeResponse)) {
            data.setApiStatus(false);
        }else{
            data.setApiStatus(true);
            if(method.equals(ApiMethodType.CREATE_ORDER.getType())) {
                CreateOrderResponseBody responseBody = (CreateOrderResponseBody) map.get(OtaUtils.DOM_BODY);
                data.setCytOrderCode(responseBody.getOrderInfo().getPartnerorderId());
            }else if(method.equals(ApiMethodType.CREATE_PAY_ORDER)){
                CreatePaymentOrderResponseBody responseBody = (CreatePaymentOrderResponseBody) map.get(OtaUtils.DOM_BODY);
                data.setCytOrderCode(responseBody.getOrderInfo().getPartnerorderId());
            }
        }
        return Result.success(data);
    }

    //<!=============================工具类或通用方法====================================!>
    public static String formatDate(Date date) {
        return DateFormatUtils.format(date, DateUtils.DATE_PATTERN);
    }

    public static long convertCents(double money) {
        return BigDecimalUtils.amplify2long(money, 2);
    }

    /*
   发送数据
    */
    private static Result<String> sendData(String url, String method, String requestJson) {
        String response = "";
        HttpClient client = new HttpClient();

        //Platfrom_sign platfrom_sign=Platfrom_sign.typeOf(platfrom_sign_type);
        //PostMethod postMethod = new UTF8PostMethod(platfrom_sign.getName() + ota_api_method_url);
        PostMethod postMethod = new UTF8PostMethod(url);

        postMethod.addParameter("Connection", "Keep-Alive");
        postMethod.addParameter("Charset", OtaUtils.ENCODE_UTF8);
        postMethod.addParameter("Content-Type", "application/x-www-form-urlencoded");
        postMethod.getParams().setParameter(HttpMethodParams.SO_TIMEOUT, 30000);//30S超时

        List<Parameter> params = new ArrayList<Parameter>();
        params.add(new Parameter("method", method));
        params.add(new Parameter("requestParam", requestJson));
        postMethod.setRequestBody(buildNameValuePair(params));

        try {
            /*java.net.URL endpointURL = new java.net.URL(url);
            CytDataServiceSoapBindingStub ssl = new CytDataServiceSoapBindingStub(endpointURL, null);
            ssl.setMaintainSession(true);
            String responseString = ssl.doCYTRequest(method, requestJson);
            return Result.success(responseString);*/
            int statusCode = client.executeMethod(postMethod);
            if (statusCode == HttpStatus.SC_OK) {
                response = postMethod.getResponseBodyAsString();
            } else {
                throw new IOException("Http Post Exception Of Code: " + statusCode);
            }
            postMethod.releaseConnection();
            client = null;

            return Result.success(response);
        }
        //PMS数据发送异常,将异常信息返回
        catch (Exception e) {
            if(e instanceof SocketTimeoutException){
                return Result.error(e, "畅游通网络超时，请稍后再试.");
            }
            return Result.error(e, "自测平台请求["+method+"]接口发生异常."+e.getMessage());
        }
    }

    private static org.apache.commons.httpclient.NameValuePair[] buildNameValuePair(List<Parameter> list) {
        int length = list.size();
        org.apache.commons.httpclient.NameValuePair[] pais = new org.apache.commons.httpclient.NameValuePair[length];
        for (int i = 0; i < length; i++) {
            Parameter param = list.get(i);
            pais[i] = new org.apache.commons.httpclient.NameValuePair(param.getName(), param.getValue());
        }
        return pais;
    }

    /*
    * 对象转XML
    */
    private static String beanToXml(RequestBody requestBody, String distributorName, String supplierIdentity) {
        try {
            RequestHeader header = new RequestHeader();
            header.setApplication("tour.ectrip.com");
            header.setProcessor("DataExchangeProcessor");
            header.setVersion("1.0.0");
            header.setCreateUser(distributorName);
            header.setCreateTime(DateUtils.formatDate(new Date(),DateUtils.DATETIME_PATTERN));
            header.setSupplierIdentity(supplierIdentity);

            return XmlParseUtil.beanToXML(requestBody, header, OtaUtils.ECTRIP);
        } catch (Throwable throwable) {
            throw new BusinessException(throwable);
        }
    }

    /*
    * XML转成map对象
    */
    private static Map xmlToBean(String xml, Object head, Object body) {
        try {
            return XmlParseUtil.xmlToBean(xml, head, body);
        }
        //XML解析异常
        catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException(e);
        }
    }

    public static Map<String, Object> parseRequest(RequestBody requestBody,
                                                    String distributorName,
                                                    String supplierIdentity,
                                                    String key,
                                                    String securityType) {
        String requestXml = beanToXml(requestBody, distributorName, supplierIdentity);

        JSONObject requestJSON = new JSONObject();

        if(StringUtils.isBlank(securityType)){
            //默认MD5加密
            securityType = OtaUtils.SECURITY_MD5;
        }
        securityType = securityType.toUpperCase();

        String requestData;
        if(securityType.equals(OtaUtils.SECURITY_MD5)){
            byte[] requestByte = requestXml.getBytes(Charset.forName(OtaUtils.ENCODE_UTF8));
            requestData = Base64Utils.encode(requestByte);

            String responseSigned = EncryptUtil.MD5Hex(key + requestData).toUpperCase();
            requestJSON.put(OtaUtils.JSON_SIGNED, responseSigned);
        }else{
            String iv = SecurityTools.generatorIv();
            if(securityType.equals(OtaUtils.SECURITY_AES_CBC)){
                requestData = AesTools.encryptCBC(requestXml,key,iv);
            }else if(securityType.equals(OtaUtils.SECURITY_SM4_CBC)){
                requestData = SM4Tools.encryptCBC(requestXml,key,iv);
            }else{
                throw new BusinessException("不支持的加密算法!");
            }
            requestJSON.put(OtaUtils.JSON_IV, iv);
            requestJSON.put(OtaUtils.JSON_CREATEUSER,distributorName);
        }

        requestJSON.put(OtaUtils.JSON_DATA, requestData);
        requestJSON.put(OtaUtils.JSON_SECURITY_TYPE,securityType);

        //返回结果
        Map<String, Object> mapResult = Maps.newHashMap();
        mapResult.put(OtaUtils.REQUEST_XML, requestXml);
        mapResult.put(OtaUtils.REQUEST_JSON_OBJECT, requestJSON);
        return mapResult;
    }

    /*
    获得返回的xml字符串
     */
    private static String getResponseXml(String responseJson, String key,String securityType) {
        JSONObject jsonObject = JSONObject.parseObject(responseJson);
        Object dataObject = jsonObject.get(OtaUtils.JSON_DATA);
        Object signedObject = jsonObject.get(OtaUtils.JSON_SIGNED);
        Object ivObject = jsonObject.get(OtaUtils.JSON_IV);
        //String securityType = jsonObject.get(OtaUtils.JSON_SECURITY_TYPE).toString();

        if(dataObject == null && dataObject.toString().equals("")){
            throw new BusinessException("data返回为空");
        }
        
        String data = dataObject.toString();
        String xml;
        if (OtaUtils.SECURITY_MD5.equals(securityType)) {
            if(signedObject == null || signedObject.toString().equals("")){
                throw new BusinessException("signed返回为空");
            }
            String signed = signedObject.toString();
            
            String signedEncryption = EncryptUtil.MD5Hex(key + data).toUpperCase();
            if (!signedEncryption.equalsIgnoreCase(signed)) {
                throw new BusinessException("签证验证不通过");
            }

            try {
                byte[] xmlByte = Base64Utils.decode(data);
                xml = new String(xmlByte,OtaUtils.ENCODE_UTF8);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                throw new BusinessException("Base64解密失败");
            }

        } else {
            if(ivObject == null || ivObject.toString().equals("")){
                throw new BusinessException("iv返回为空");
            }
            String iv = ivObject.toString();

            if (securityType.equals(OtaUtils.SECURITY_AES_CBC)) {
                xml = AesTools.decryptCBC(data,key,iv);
            }else if (securityType.equals(OtaUtils.SECURITY_SM4_CBC)) {
                xml = SM4Tools.decryptCBC(data,key,iv);
            }else {
                throw new BusinessException("不支持的加密算法");
            }
        }
        return xml;
    }


}
