package com.ectrip.cyt.controller;

import com.alibaba.fastjson.JSONObject;
import com.ectrip.cyt.client.cyt.service.TdosApiClient;
import com.ectrip.cyt.model.*;
import com.ectrip.cyt.model.Request;
import com.ectrip.cyt.model.param.AccountParam;
import com.ectrip.cyt.model.param.CreateOrderParam;
import com.ectrip.cyt.model.param.ProductParam;
import com.ectrip.cyt.model.pojo.Person;
import com.ectrip.cyt.model.result.AjaxResult;
import com.ectrip.cyt.model.result.Result;
import com.ectrip.cyt.model.rt.CommonReturn;
import com.ectrip.cyt.model.rt.PushOrderRequest;
import com.ectrip.cyt.utils.DateUtils;
import com.ectrip.cyt.utils.UUIDTools;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.validation.Valid;
import java.util.*;

@RestController
@RequestMapping(value = "/tdos")
@Api(value = "整合营销OTA自测接口",tags = {"整合营销OTA自测接口"})
public class TdosApiController {
    @ApiOperation(value = "心跳检测接口",notes = "心跳检测接口")
    @ResponseBody
    @PostMapping("/heartBeat")
    public AjaxResult heartBeat(@RequestBody @Valid Request request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.heartBeat(request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }


    @ApiOperation(value = "查询用户余额",notes = "获取产品接口")
    @ResponseBody
    @PostMapping("/getBalance")
    public AjaxResult getBalance(@RequestBody @Valid Request request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            //判断方法类型
            StringBuffer errorMsg=new StringBuffer();
            if(StringUtils.isNotBlank(errorMsg)){
                ajaxResult.setDescription(errorMsg.toString());
                return ajaxResult;
            }
            Result result=TdosApiClient.getBalance(request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }

    @ApiOperation(value = "获取景区接口",notes = "获取景区接口")
    @ResponseBody
    @PostMapping("/queryPois")
    public AjaxResult queryPois(@RequestBody @Valid Request request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.queryPois(request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }


    @ApiOperation(value = "获取产品接口",notes = "获取产品接口")
    @ResponseBody
    @PostMapping("/queryProducts")
    public AjaxResult queryProducts(@RequestBody @Valid GetProductRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            //判断方法类型
            StringBuffer errorMsg=new StringBuffer();
            if(StringUtils.isBlank(request.getQueryProducts().getResourceId())){
                if(request.getQueryProducts().getCurrentPage()==null){
                    errorMsg.append("查询所有可分销商品时,未设置页数");
                }else{
                    if(request.getQueryProducts().getCurrentPage()<=0){
                        errorMsg.append("查询所有可分销商品时,设置页数必须大于0");
                    }
                }
                if(request.getQueryProducts().getPageSize()==null){
                    errorMsg.append("查询所有可分销商品时,未设置每页条数");
                }else{
                    if(request.getQueryProducts().getPageSize()>5||request.getQueryProducts().getPageSize()<=0){
                        errorMsg.append("查询所有可分销商品时,每页条数范围在[1,5]内");
                    }
                }
            }

            if(StringUtils.isNotBlank(errorMsg)){
                ajaxResult.setDescription(errorMsg.toString());
                return ajaxResult;
            }

            /*if(request.getQueryProducts().getMethod().equals("SINGLE")){
                if(!StringUtils.isBlank(request.getQueryProducts().getResourceId())){
                    ajaxResult.setDescription("获取产品方法为单个时, 产品ID必填");
                    return ajaxResult;
                }
            }*/
            Result result=TdosApiClient.queryProducts(request.getQueryProducts(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }

    @ApiOperation(value = "获取库存接口",notes = "获取库存接口")
    @ResponseBody
    @PostMapping("/queryStock")
    public AjaxResult queryStock(@RequestBody @Valid GetStockRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.queryStockAndTsds(request.getGetStockParam(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }

    /*@ApiOperation(value = "获取座位接口",notes = "获取座位接口")
    @ResponseBody
    @PostMapping("/querySeatList")
    public AjaxResult querySeatList(@RequestBody @Valid GetSeatRequset request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.querySeatList(request.getGetSeatParam(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }*/

    @ApiOperation(value = "重发短信接口",notes = "重发短信接口")
    @ResponseBody
    @PostMapping("/sendOrderEticket")
    public AjaxResult sendOrderEticket(@RequestBody @Valid SendOrderEticketRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.sendOrderEticket(request.getParam(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }

    @ApiOperation(value = "下单接口",notes = "下单接口")
    @ResponseBody
    @PostMapping("/createOrder")
    public AjaxResult createOrder(@RequestBody @Valid CreateOrderRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.createOrder(request.getCreateOrder(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }


    @ApiOperation(value = "查询订单接口",notes = "查询订单接口")
    @ResponseBody
    @PostMapping("/queryOrder")
    public AjaxResult queryOrder(@RequestBody @Valid QueryOrderRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            //判断订单状态
            Result result=TdosApiClient.queryOrder(request.getQueryOrder(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }


    @ApiOperation(value = "改签接口",notes = "改签接口")
    @ResponseBody
    @PostMapping("/pushOrder")
    public AjaxResult pushOrder(@RequestBody @Valid PushOrderRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.pushOrder(request.getPushOrder(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }

    @ApiOperation(value = "申请退订接口",notes = "申请退订接口")
    @ResponseBody
    @PostMapping("/refundOrder")
    public AjaxResult refundOrder(@RequestBody @Valid RefundOrderRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.applyOrderRefundByUser(request.getRefundOrder(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }

    @ApiOperation(value = "取消订单接口",notes = "取消订单接口")
    @ResponseBody
    @PostMapping("/cancelOrder")
    public AjaxResult cancelOrder(@RequestBody @Valid CancelOrderRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.cancelOrder(request.getCancelOrder(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }

    @ApiOperation(value = "支付接口",notes = "支付接口")
    @ResponseBody
    @PostMapping("/payOrder")
    public AjaxResult payOrder(@RequestBody @Valid PayOrderRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.payOrder(request.getPayOrder(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }

    @ApiOperation(value = "下单前校验接口",notes = "下单前校验接口")
    @ResponseBody
    @PostMapping("/checkOrder")
    public AjaxResult checkOrder(@RequestBody @Valid CreateOrderRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.checkOrder(request.getCreateOrder(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }

    @ApiOperation(value = "下单并支付接口",notes = "下单并支付接口")
    @ResponseBody
    @PostMapping("/createPaymentOrder")
    public AjaxResult createPaymentOrder(@RequestBody @Valid CreateOrderRequest request, BindingResult results) {
        AjaxResult ajaxResult = new AjaxResult();
        ajaxResult.setSuccess(false);
        try {
            if (results.hasErrors()) {
                ajaxResult.setDescription("请求参数错误:"+results.getFieldError().getDefaultMessage());
                return ajaxResult;
            }
            Result result=TdosApiClient.createPaymentOrder(request.getCreateOrder(),request.getAccount());
            if(result.isSuccess()) {
                CommonReturn commonReturn=(CommonReturn)result.getData();
                ajaxResult.setSuccess(true);
                ajaxResult.setDescription("接口联通成功");
                ajaxResult.setData(commonReturn);
            }else{
                ajaxResult.setDescription("接口联通失败,错误原因:"+result.getDescription());
            }
        } catch (Exception e) {
            e.printStackTrace();
            ajaxResult.setDescription("接口联通异常!"+e.getMessage());
        }
        return ajaxResult;
    }
}
