package com.ectrip.cyt.model.enums;

import java.util.HashMap;
import java.util.Map;

/**
 * 游客证件类型枚举：(身份证 : ID_CARD,护照 : HUZHAO,台胞证 : TAIBAO ，港澳通行证: GANGAO 其它：OTHER)
 * Created by weiyu.huang on 2015/9/2.
 */
public enum CredentialsType {

    /**
     * 身份证
     */
    ID_CARD(0, "身份证"),

    /**
     * 护照
     */
    HUZHAO(1, "护照 PASSPORT"),

    /**
     * 军官证
     */
    OFFICER(2, "军官证"),

    /**
     * 台胞证
     */
    TAIBAO(3, "台胞证"),

    /**
     * 港澳通行证
     */
    GANGAO(4, "港澳通行证"),

    /**
     * 其它
     */
    OTHER(5, "其它"),

    /**
     * 皈依证
     */
    GUIYI(6, "皈依证"),

    /**
     * 学生证
     */
    STUDENT(7,"学生证");

    /**
     * 类型
     */
    private int type;

    /**
     * 名称
     */
    private String name;

    /**
     * 构造函数，枚举类型构造函数只能私有
     */
    CredentialsType(int type, String name) {
        this.type = type;
        this.name = name;
    }

    public int getType() {
        return this.type;
    }

    public String getName() {
        return this.name;
    }

    /**
     * 根据类型获取枚举对象
     *
     * @param type 枚举类型值
     * @return 枚举对象
     */
    public static CredentialsType typeOf(int type) {
        for(CredentialsType credentialsType : values()) {
            if(credentialsType.getType() == type) {
                return credentialsType;
            }
        }
        throw new IllegalArgumentException("Invalid CredentialsType type: " + type);
    }

    /**
     * 枚举转成 Map
     * @return Map
     */
    public static Map<Integer, Object> convertToMap() {
        Map<Integer, Object> map = new HashMap<Integer, Object>();
        for(CredentialsType credentialsType : values()) {
            map.put(credentialsType.getType(), credentialsType.getName());
        }
        return map;
    }

    /**
     * 判断是否存在该证件类型值
     * @param type 证件类型值
     * @return true/false
     */
    public static boolean contains(int type) {
        return CredentialsType.convertToMap().containsKey(type);
    }

    @Override
    public String toString() {
        return "CredentialsType{" +
                "type=" + type +
                ", name='" + name + '\'' +
                '}';
    }
}
