package com.ectrip.cyt.model.enums;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 订单状态
 * @author Jiehao Ling, date:9/26/2015, time:11:24 AM
 */
public enum OrderStatusType {

    /** 订单状态 */
    ORDER_NOT_PAYED_YET ( 0 , "未付款"),

    ORDER_CANCELED ( 1 , "已取消"),

    ORDER_WAITING_CONFIRM ( 2 , "待确认"),

    ORDER_RESERVE_FAILED ( 3 , "预订失败"),

    ORDER_RESERVE_SUCCESSFUL ( 4 , "预订成功"),

    ORDER_CONSUMED(5, "已消费"),

    ORDER_REFUNDED(6, "已退款"),

    ORDER_CONSUME_CONFIRM(7,"消费待确认"),

    ORDER_REFUNDED_CONFIRM(8,"退订待确认"),

    ORDER_RESERVE_CONFIRM(9,"订单待确认"),

    ORDER_OVERDUE(10,"订单过期"),

    ORDER_EXCEPTION(11,"订单异常"),

    ORDER_REFUNDED_WAIT_AUDIT(12,"退订待审核"),

    LMM_ORDER_PUSH(13,"预定成功待推送"),//驴妈妈酒店订单异步用

    ORDER_NOT_PUSH_SUCCESS(14,"订单未推送成功或已全退");//驴妈妈酒店订单异步用

    OrderStatusType(int statusIndex, String statusName) {
        this.statusIndex = statusIndex;
        this.statusName = statusName;
    }

    /** 状态下标 */
    private int statusIndex;

    /** 状态名称 */
    private String statusName;

    public int getStatusIndex() {
        return statusIndex;
    }

    public void setStatusIndex(int statusIndex) {
        this.statusIndex = statusIndex;
    }

    public String getStatusName() {
        return statusName;
    }

    public void setStatusName(String statusName) {
        this.statusName = statusName;
    }

    /**
     * 根据订单状态ID类型获取枚举对象
     * @param orderStatusIndex 支付类型值
     * @return 支付类型枚举
     */
    public static OrderStatusType typeOf(int orderStatusIndex){
        OrderStatusType[] OrderStatusTypes = OrderStatusType.values();
        for(OrderStatusType OrderStatusType: OrderStatusTypes){
            if(OrderStatusType.getStatusIndex() == orderStatusIndex){
                return OrderStatusType;
            }
        }
        throw new IllegalArgumentException("非法订单类型: " + orderStatusIndex);
    }

    /**
     * 枚举转Map
     * @return Map
     */
    public static Map<Integer,String> convertToMap(){
        Map<Integer,String> OrderStatusTypeMap = new HashMap<Integer, String>();
        OrderStatusType[] OrderStatusTypes = OrderStatusType.values();
        for(OrderStatusType OrderStatusType : OrderStatusTypes){
            OrderStatusTypeMap.put(OrderStatusType.getStatusIndex(),OrderStatusType.getStatusName());
        }
        return OrderStatusTypeMap;
    }

    /**
     * 枚举转换成JSON字符串
     * @author yongkang.liao
     * @date 2015-12-03
     * @return String
     */
    public static String convertJson(){
        OrderStatusType[] orderStatusTypes = OrderStatusType.values();
        StringBuffer sb = new StringBuffer();
        sb.append("{\"orderStatus\":[");
        for(int i = 0; i < orderStatusTypes.length; i++){
            OrderStatusType orderStatusType = orderStatusTypes[i];
            sb.append("{\"type\":" + orderStatusType.getStatusIndex() + ",\"name\":\""
                    + orderStatusType.getStatusName() + "\"}");
            if((i + 1) != orderStatusTypes.length){
                sb.append(",");
            }
        }
        sb.append("]}");
        return sb.toString();
    }

    @Override
    public String toString() {
        return "OrderStatusType{" +
                "statusIndex=" + statusIndex +
                ", statusName='" + statusName + '\'' +
                '}';
    }
}
