package com.ectrip.cyt.model.enums;

import java.util.HashMap;
import java.util.Map;

/**
 * 支付类型枚举
 * @author wenqiang.luo date:15-9-3
 */
public enum PaymentWay {

    /**
     * 在线支付
     */
    PRE_PAY(0, "预付"),

    /**
     * 现场支付
     */
    CASH_PAY(1, "到付");

    /** 支付类型 */
    private int way;

    /** 支付名称 */
    private String name;

    PaymentWay(int way, String name) {
        this.way = way;
        this.name = name;
    }

    public int getWay() {
        return way;
    }

    public String getName() {
        return name;
    }

    /**
     * 根据支付类型获取枚举对象
     * @param type 支付类型值
     * @return 支付类型枚举
     */
    public static PaymentWay wayOf(int type) {
        //枚举结果
        PaymentWay[] paymentWays = PaymentWay.values();
        for (PaymentWay paymentWay : paymentWays) {
            if (paymentWay.getWay() == type) {
                return paymentWay;
            }
        }
        throw new IllegalArgumentException("Invalid PaymentType way: " + type);
    }

    /**
     * 枚举转Map
     * @return Map
     */
    public static Map<Integer, String> convertToMap() {
        Map<Integer, String> integerStringMap = new HashMap<Integer, String>();
        PaymentWay[] paymentWays = PaymentWay.values();
        for (PaymentWay paymentWay : paymentWays) {
            integerStringMap.put(paymentWay.getWay(), paymentWay.getName());
        }
        return integerStringMap;
    }

    @Override
    public String toString() {
        return "PaymentWay{" +
                "way=" + way +
                ", name='" + name + '\'' +
                '}';
    }
}
