package com.ectrip.cyt.model.param;

import com.ectrip.cyt.model.pojo.Person;
import com.ectrip.cyt.utils.DateUtils;
import com.ectrip.cyt.utils.TestOrderUtils;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.validator.constraints.Range;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Data
@AllArgsConstructor
@NoArgsConstructor
@ApiModel(description = "创建订单参数")
public class CreateOrderParam {
    //ota订单号
    //@NotBlank(message = "OTA订单号不能为空")
    @ApiModelProperty(value = "OTA订单号", required = true)
    private String orderCode;

    public String getOrderCode() {
        if(StringUtils.isBlank(orderCode)){
            orderCode = TestOrderUtils.get16UUID();
        }
        return orderCode;
    }

    //游览日期
    //@NotBlank(message = "游览日期不能为空")
    //@Pattern(regexp = "(([0-9]{3}[1-9]|[0-9]{2}[1-9][0-9]{1}|[0-9]{1}[1-9][0-9]{2}|[1-9][0-9]{3})-(((0[13578]|1[02])-(0[1-9]|[12][0-9]|3[01]))|((0[469]|11)-(0[1-9]|[12][0-9]|30))|(02-(0[1-9]|[1][0-9]|2[0-8]))))|((([0-9]{2})(0[48]|[2468][048]|[13579][26])|((0[48]|[2468][048]|[3579][26])00))-02-29)",message = "游览日期格式不对")
    @ApiModelProperty(value = "住店日期", required = true)
    private String visitDate;

    @ApiModelProperty(value = "离店日期", required = true)
    private String leaveDate;

    public String getVisitDate() {
        if(StringUtils.isBlank(visitDate)){
            visitDate = DateUtils.formatDate(new Date(),DateUtils.DATE_PATTERN);
        }
        return visitDate;
    }

    //票数
    @Min(value = 1,message = "订单数量必须大于或等于1")
    @ApiModelProperty(value = "订单数量", required = true)
    private Integer orderQuantity;

    //金额
    @Min(value = 0,message = "订单金额必须大于或等于0")
    @ApiModelProperty(value = "订单金额", required = true)
    private Double orderPrice;

    @ApiModelProperty(value = "一票几人 默认一票一人", required = false)
    private Integer ticketPersonNum;

    @Valid
    @NotNull(message = "必须传入产品信息")
    @ApiModelProperty(value = "产品信息", required = true)
    //产品信息
    private ProductParam product;

    //@NotNull(message = "必须传入联系人信息 至少保留手机号")
    @ApiModelProperty(value = "联系人信息", required = true)
    //联系人
    private Person contactPerson;

    @ApiModelProperty(value = "游客信息", required = false)
    //游客信息
    private List<Person> visitPersons;

    public Person getContactPerson() {
        if(contactPerson == null){
            contactPerson = new Person();
        }
        if(StringUtils.isBlank(contactPerson.getName())){
            contactPerson.setName("测试联系人");
        }
        if(contactPerson.getCredentialsType() == null){
            contactPerson.setCredentialsType(0);
        }
        if(StringUtils.isBlank(contactPerson.getPhone())){
            contactPerson.setPhone(TestOrderUtils.createMobile(2));
        }
        if(StringUtils.isBlank(contactPerson.getCredentials())){
            contactPerson.setCredentials(TestOrderUtils.getIdNo(false));
        }
        return contactPerson;
    }

    public List<Person> getVisitPersons() {
        if(CollectionUtils.isEmpty(visitPersons)){
            visitPersons = new ArrayList<>();
            visitPersons.add(contactPerson);

            if(ticketPersonNum == null){
                ticketPersonNum = 1;
            }

            if((ticketPersonNum * orderQuantity) >1) {
                for (int i = 0; i < ((ticketPersonNum * orderQuantity) - 1); i++) {
                    Person p = new Person();
                    p.setName("测试游客" + (i+1));
                    p.setCredentialsType(0);
                    p.setPhone(TestOrderUtils.createMobile(2));
                    p.setCredentials(TestOrderUtils.getIdNo(false));
                    visitPersons.add(p);
                }
            }
        }
        return visitPersons;
    }
}
