package com.ectrip.cyt.model.result;

import com.ectrip.cyt.model.rt.CommonReturn;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;

/**
 * 处理ajax返回的结果信息
 * @author wenqiang.luo date:15-9-7
 */
public class AjaxResult implements Serializable {

    private static final long serialVersionUID = 5837409853066710012L;

    private static final String SUCCESS_DESCRIPTION = "success";

    /** ajax 交互结果 */
    @ApiModelProperty(value = "请求状态", required = true)
    private boolean success;

    /** 交互描述信息 */
    @ApiModelProperty(value = "请求描述", required = true)
    private String description;

    /** 交互数据 */
    @ApiModelProperty(value = "接口返回信息", required = true)
    private CommonReturn data;


    //construct////////////////////////////////////////////////////////////////

    public AjaxResult() {
        this(false);
    }

    public AjaxResult(boolean success) {
        this(success, SUCCESS_DESCRIPTION);
    }

    public AjaxResult(boolean success, String description) {
        this(success, description, null);
    }

    public AjaxResult(boolean success, String description, CommonReturn data) {
        this.success = success;
        this.description = description;
        this.data = data;
    }

    //get and set//////////////////////////////////////////////////////////////

    public boolean isSuccess() {
        return success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public CommonReturn getData() {
        return data;
    }

    public void setData(CommonReturn data) {
        this.data = data;
    }


    //public static method/////////////////////////////////////////////////////

    public static AjaxResult error(String description) {
        return new AjaxResult(false, description);
    }

    public static AjaxResult success(String description) {
        return new AjaxResult(true, description);
    }

    public static AjaxResult success(CommonReturn data) {
        return new AjaxResult(true, null, data);
    }

    public static AjaxResult success(String description, CommonReturn data) {
        return new AjaxResult(true, description, data);
    }

}
