package com.ectrip.cyt.model.result;

import java.io.Serializable;

/**
 * @author wenqiang.luo date:16/7/28
 */
public class Result<T> implements Serializable {

    private static final long serialVersionUID = -2336951543661721293L;

    /** 交互成功标识 */
    private boolean success;

    /** 交互异常信息 */
    private Throwable throwable;

    /** 结果描述 */
    private String description;

    /** 附加信息 */
    private T data;

    //constructor//////////////////////////////////////////////////////////////

    private Result(boolean success, Throwable throwable, String description, T data) {
        this.success = success;
        this.description = description;
        this.throwable = throwable;
        this.data = data;
    }

    //public static method/////////////////////////////////////////////////////

    public static <T> Result<T> success() {
        return Result.success(null);
    }

    public static <T> Result<T> success(T data) {
        return Result.success("SUCCESS", data);
    }

    public static <T> Result<T> success(String description, T data) {
        return new Result<T>(true, null, description, data);
    }

    public static <T> Result<T> error(String description) {
        return new Result<T>(false, null, description, null);
    }

    public static <T> Result<T> error(String description,T data) {
        return new Result<T>(false, null, description, data);
    }

    public static <T> Result<T> error(Throwable throwable, String description) {
        return new Result<T>(false, throwable, description, null);
    }

    //get and set//////////////////////////////////////////////////////////////

    public boolean isSuccess() {
        return success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    public Throwable getThrowable() {
        return throwable;
    }

    public void setThrowable(Throwable throwable) {
        this.throwable = throwable;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }

    @Override
    public String toString() {
        return "Result{" +
                "success=" + success +
                ", description='" + description + '\'' +
                ", data=" + data +
                '}';
    }
}
