package com.ectrip.cyt.utils;

import com.ectrip.cyt.exception.BusinessException;
import org.apache.commons.codec.binary.Base64;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.security.NoSuchAlgorithmException;
import java.security.Security;

/**
 * @author chenys
 * @date 2022-12-06
 */
public class AesTools {
    private static final String ALGORITHM = "AES";

    /**
     * AES算法
     */
    private static final String AES_CBC_PCK_ALG = "AES/CBC/PKCS5Padding";

    // 128-32位16进制；256-64位16进制
    public static final int DEFAULT_KEY_SIZE = 256;


    public static String generateKey() throws Exception {
        KeyGenerator kg = KeyGenerator.getInstance(ALGORITHM);
        //下面调用方法的参数决定了生成密钥的长度，可以修改为128, 192或256
        kg.init(DEFAULT_KEY_SIZE);
        SecretKey sk = kg.generateKey();
        byte[] b = sk.getEncoded();
        String secret = Base64.encodeBase64String(b);
        return secret;
    }



    /**
     * AES加密
     *
     * @param content
     * @param aesKey
     * @return
     * @throws Exception
     */
    public static String encryptCBC(String content, String aesKey,String ivBase64){
        try {
            Cipher cipher = Cipher.getInstance(AES_CBC_PCK_ALG);
            IvParameterSpec ivParameterSpec = new IvParameterSpec(SecurityTools.decodeIv(ivBase64));
            cipher.init(Cipher.ENCRYPT_MODE, new SecretKeySpec(Base64.decodeBase64(aesKey.getBytes()), ALGORITHM), ivParameterSpec);
            byte[] encryptBytes = cipher.doFinal(content.getBytes(OtaUtils.ENCODE_UTF8));
            return new String(Base64.encodeBase64(encryptBytes));
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("AES加密失败");
        }
    }

    /**
     * AES解密
     *
     * @param content
     * @param key
     * @return
     * @throws Exception
     */
    public static String decryptCBC(String content, String key,String ivBase64) {
        try {
            Cipher cipher = Cipher.getInstance(AES_CBC_PCK_ALG);
            IvParameterSpec ivParameterSpec = new IvParameterSpec(SecurityTools.decodeIv(ivBase64));
            cipher.init(Cipher.DECRYPT_MODE, new SecretKeySpec(Base64.decodeBase64(key.getBytes()), ALGORITHM), ivParameterSpec);
            byte[] cleanBytes = cipher.doFinal(Base64.decodeBase64(content.getBytes()));
            return new String(cleanBytes, OtaUtils.ENCODE_UTF8);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("AES解密失败");
        }
    }
}
